/*
Top module
*/

module text_gen(
    //Common
    input wire clk,
    input wire rst,
    //HDMI
	output wire[2:0] TMDSp,
	output wire TMDSp_clock,
	//VRAM Inteface
	input wire[7:0]AD,
	input wire[2:0]CMD,
	input wire strobe
    );

//Clocks
wire HDMI_CLK; //250MHz
wire PIX_CLK; //25MHz
//
wire[9:0]HCNT,VCNT; //H/V Counters
wire[7:0]R,G,B;
wire VISIBLE;
//VRAM
wire[13:0]VADDR,VADDW;
wire[7:0]VDATAI,VDATAO;
wire VCLKR,VCLKW,VWREN;

reg[7:0]data;
reg[13:0]add;
reg[1:0]chbnk;
wire nowr;


main_pll mpl0(.inclk0(clk),.c0(PIX_CLK),.c1(HDMI_CLK));
hdmi mhd0(.n_rst(rst),.pixclk(PIX_CLK),.clk_TMDS(HDMI_CLK),.TMDSp(TMDSp),.TMDSp_clock(TMDSp_clock),
				.HCNT(HCNT),.VCNT(VCNT),.red(R),.green(G),.blue(B),.visible(VISIBLE));
				
videoproc mvp0(.clk(PIX_CLK),.clk50(clk),.rst(rst),.HCNT(HCNT),.VCNT(VCNT),.visible(VISIBLE),.R(R),.G(G),.B(B),
					.VADD(VADDR),.VDATA(VDATAO),.VCLK(VCLKR),.chbank(chbnk));

vram mvr0(.rdaddress(VADDR),.q(VDATAO),.rdclock(VCLKR),.wren(VWREN),.data(VDATAI),.wraddress(VADDW),
			.wrclock(VCLKW));
			
//Logika upravleniya zapis'yu VRAM

initial
	begin
		chbnk[1:0] = 2'b01;
	end


always@(posedge strobe or negedge rst)
begin
	if(!rst)
		begin
			data <= 0;
			add <= 0;
		end
	else
		begin
			case(CMD)
				3'b000: data[7:0] <= AD[7:0]; //Защёлкиваем данные
				3'b001: add[7:0] <= AD[7:0];  //Защёлкиваем младший байт адреса
				3'b010: add[13:8] <= AD[5:0]; //Защёлкиваем старший байт адреса
				3'b100: if(AD[1:0]) chbnk[1:0] <= AD[1:0]; //Защёлкиваем номер банка CHRAM, если он больше нуля
				default:;
			endcase
		end
end



assign VADDW[13:0] = add[13:0];
assign VDATAI[7:0] = data[7:0];
assign VWREN = CMD[0] & CMD[1] & ~CMD[2];
//assign nowr = (VADDW[12:11] == 2'b01)? 0 : 1;
assign VCLKW = ~(strobe);

endmodule
