module videoproc(
	input wire clk,
	input wire clk50,
	input wire rst,
	//CHRAM bank
	input wire[1:0]chbank,
	//HDMI Signals
	input wire[9:0]HCNT,
	input wire[9:0]VCNT,
	input wire visible,
	output wire[7:0]R,
	output wire[7:0]G,
	output wire[7:0]B,
	//VIDEO RAM
	output wire[13:0]VADD,
	output wire VCLK,
	input wire[7:0]VDATA
);

//Vychislenie znakomesta
wire[12:0]znakomesto;

reg[9:0]hc;
wire hvis;

assign hvis = (HCNT >= 152 && HCNT < 792)? 1'b1 : 1'b0;

always@(posedge clk)
begin
	if(hvis) hc <= hc + 1;
	else hc <= 0;
end

assign znakomesto = hc[9:3] + (VCNT[9:3] * 80);


//Avtomat chteniya iz videopamyati
reg vclk;
reg[3:0]cnt;
reg[7:0]znad;
reg[13:0]cadd;
reg[7:0]tshift, shift;
reg[7:0]tattr, attr;

always@(posedge clk50)
begin
	if(hvis) cnt <= cnt + 1;
	else cnt <= 0;
end


always@(cnt)
begin
	case(cnt[3:0])
		2: cadd <= znakomesto;
		3: vclk <= 1;
		4: vclk <= 0;
		5: znad <= VDATA;
		6: cadd <= znakomesto + 4800;
		7: vclk <= 1;
		8: vclk <= 0;
		9: tattr <= VDATA;
		10: cadd[13:0] <= {1'b1,chbank[1:0],znad[7:0], VCNT[2:0]}; //Добавить переключение банков CHAR cadd[12:11]
		11: vclk <= 1;
		12: vclk <= 0;
		13: tshift <= VDATA;
		default:
			begin
				vclk <= 0;
			end
	endcase
end


assign VCLK = ~(vclk & hvis);
assign VADD = cadd;

always@(posedge clk) 
begin
	if(HCNT[2:0] == 7 && hvis) 
		begin 
			shift <= tshift;
			attr <= tattr;
		end
end

//Palitra 16 Color
reg[23:0]colors[0:15];

initial 
	begin
		colors[0] = 24'hFFFFFF; //Белый
		colors[1] = 24'hFFFF00; //Желтый
		colors[2] = 24'h00FF00; //Салатовый
		colors[3] = 24'h808000; //Оливковый
		colors[4] = 24'h008000; //Зелёный
		colors[5] = 24'h008080; //Сине-серый
		colors[6] = 24'h000080; //Тёмно-синий
		colors[7] = 24'h0000FF; //Синий
		colors[8] = 24'h00FFFF; //Голубой
		colors[9] = 24'hFF00FF; //Сиреневый
		colors[10] = 24'h800080; //Фиолетовый
		colors[11] = 24'hFF0000; //Красный
		colors[12] = 24'h800000; //Каштановый
		colors[13] = 24'h000000; //Чёрный
		colors[14] = 24'h808080; //Серый
		colors[15] = 24'hC0C0C0; //Серебро
	end

//Vyvod izobrazheniya
wire[7:0]Rg,Gg,Bg;

assign Rg[7:0] = (shift[(HCNT[2:0])] == 1)? colors[(attr[7:4])][23:16] : colors[(attr[3:0])][23:16];
assign Gg[7:0] = (shift[(HCNT[2:0])] == 1)? colors[(attr[7:4])][15:8] : colors[(attr[3:0])][15:8];
assign Bg[7:0] = (shift[(HCNT[2:0])] == 1)? colors[(attr[7:4])][7:0] : colors[(attr[3:0])][7:0];

assign R[7:0] = (visible)? Rg[7:0] : 8'h00;
assign G[7:0] = (visible)? Gg[7:0] : 8'h00;
assign B[7:0] = (visible)? Bg[7:0] : 8'h00;

endmodule

